/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FR231x Demo: Temperature Sensing with PTC Circuit + ADC10
//
//  Description: Initialize SAC0 in general purpose mode and connect the 
//  appropriate external circuitry to realize the temperature sensing circuit. 
//  The reference voltage for the temperature sensing circuit can come from a 
//  resistor divider or an external source, such as a DAC. SAC is configured to 
//  low power mode. Output of the temperature sensing circuit is sampled by the 
//  ADC10 and the status LED turns on if the measured temperature is over 
//  25 deg C ( 1.67 V).
//  ACLK = n/a, MCLK = SMCLK = default DCODIV ~1MHz.
//
//                MSP430FR231x
//             -------------------
//         /|\|          P1.2/OA0-|----||--------R2=1Kohm<---Vref
//          | |                   | R3=11.7Kohm
//          | |       P1.3/OA0O/A3|----||------>Vout             VCC
//          --|RST                |                               ||
//            |          P1.4/OA0+|<---||-------------R1=10KOhm---||
//            |                   |    ||
//            |                   |  PTC Thermistor
//            |                   |    ||
//            |                   |    GND
//            |                   |
//            |               P1.0|----> LED
//
//   Matt Calvo
//   Texas Instruments Inc.
//   June 2019
//   Built with IAR Embedded Workbench v7.12.1 & Code Composer Studio v9.0.1 
//******************************************************************************
#include <msp430.h>

unsigned int adcResult;

int main(void)
{
    WDTCTL = WDTPW + WDTHOLD;                // Stop watch dog timer

    P1DIR |= BIT0;                           // Select P1.0 as output
    P1OUT &= ~BIT0;                          // Set P1.0 output low
    // Disable the GPIO power-on default high-impedance mode
    // to activate previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;  
  
    //Configure SAC0 to realize the PTC thermistor circuit  
    P1SEL0 |= BIT2 + BIT3 + BIT4;            // Select P1.2(OA0-), P1.3(OAO), 
                                             // and P1.4(OA0+) as OA function
    P1SEL1 |= BIT2 + BIT3 + BIT4;            // Select P1.2(OA0-), P1.3(OAO/A3), 
                                             // and P1.4(OA0+) as OA function
    SAC0OA |= NMUXEN + PMUXEN;               // Enable SAC0 negative and 
                                             // positive input
    SAC0OA |= OAPM;                          // Select low speed and low power mode
    SAC0OA |= SACEN + OAEN;                  // Enable SAC0 and OA
  
    // Configure ADC10
    ADCCTL0 &= ~ADCENC;                      // Disable ADC
    ADCCTL0 = ADCSHT_2 | ADCON;              // ADCON, S&H=16 ADC clks, single
                                             // conversion
    ADCCTL1 = ADCSHP;                        // ADCCLK = MODOSC; sampling timer
    ADCCTL2 = ADCRES;                        // 10-bit conversion results
    ADCIE = ADCIE0;                          // Enable ADC conv complete interrupt
    ADCMCTL0 = ADCINCH_3 | ADCSREF_0;        // A3 ADC input select = OA output
                                             // Vref = DVCC

    while(1)
    {
        ADCCTL0 |= ADCENC | ADCSC;           // Sampling and conversion start
        __bis_SR_register(LPM0_bits | GIE);  // Enter LPM0, ADC_ISR will force exit
        __no_operation();                     // For debug only

        // Given DVCC = 3.3V: 1.67V/3.3V = 0.5061
        // With a 10-bit ADC: 0.5061 * 2^10 = 518
        if (adcResult > 518)                  // OA output > 1.67V / 25 deg C 
        {
            P1OUT |= BIT0;                    // Set P1.0 LED on
        }
        else
        {
            P1OUT &= ~BIT0;                   // Clear P1.0 LED off
        }
    }
}

// ADC interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=ADC_VECTOR
__interrupt void ADC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC_VECTOR))) ADC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(ADCIV, ADCIV_ADCIFG))
    {
        case ADCIV_NONE:
            break;
        case ADCIV_ADCOVIFG:
            break;
        case ADCIV_ADCTOVIFG:
            break;
        case ADCIV_ADCHIIFG:
            break;
        case ADCIV_ADCLOIFG:
            break;
        case ADCIV_ADCINIFG:
            break;
        case ADCIV_ADCIFG:
            adcResult = ADCMEM0;                 // Read ADC memory
            __bic_SR_register_on_exit(LPM0_bits);// Exit from LPM
            break;
        default:
            break;
    }
}