/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FR235x Demo: Temperature Sensing with PTC Circuit + ADC12
//
//  Description: Configure SAC2 in DAC mode to provide the reference voltage
//  for the temperature sensing circuit. Configure SAC0 in general purpose mode 
//  to connect the appropriate external circuitry. The output of SAC2 (OA2O) is
//  routed to provide the reference voltage connected to R2 and the passive
//  components are connected to OA0- and OA0+ to implement temperature sensing.
//  SACs are configured to low power mode. Output of the temperature sensing
//	circuit is sampled by the ADC12 and the status LED turns on if the measured
// 	temperature is over 25 deg C ( 1.67 V).
//  ACLK = n/a, MCLK = SMCLK = default DCODIV ~1MHz.
//
//                MSP430FR235x
//             -------------------
//         /|\|          P1.2/OA0-|----||---------R2=1Kohm<---Vref(from OA2O)
//          | |                   | R3=11.7Kohm
//          | |       P1.1/OA0O/A1|----||------>Vout              VCC
//          --|RST                |                               ||
//            |          P1.3/OA0+|<---||-------------R1=10KOhm---||
//            |                   |    ||
//            |                   |  PTC Thermistor
//            |                   |    ||
//            |                   |    GND
//            |                   |
//            |          P3.1/OA2O|--->Vref
//            |          P3.2/OA2-|
//            |          P3.3/OA2+|
//            |                   |
//            |               P1.0|----> LED
//
//   Matt Calvo
//   Texas Instruments Inc.
//   June 2019
//   Built with IAR Embedded Workbench v7.12.1 & Code Composer Studio v9.0.1 
//******************************************************************************
#include <msp430.h>

unsigned int adcResult;

int main(void)
{
    WDTCTL = WDTPW + WDTHOLD;                 // Stop watch dog timer

    P1DIR |= BIT0;                            // Select P1.0 as output
    P1OUT &= ~BIT0;                           // Set P1.0 output low
    PM5CTL0 &= ~LOCKLPM5;                     // Disable the GPIO power-on default 
                                              // high-impedance mode
                                              // to activate previously configured 
                                              // port settings  
    
    // Configure reference module
    PMMCTL0_H = PMMPW_H;                      // Unlock the PMM registers
    PMMCTL2 = INTREFEN | REFVSEL_2;           // Enable internal 2.5V reference
    while(!(PMMCTL2 & REFGENRDY));            // Poll till internal reference settles 
  
    //Configure SAC2 to provide the PTC circuit reference voltage
    P3SEL0 |= BIT1;                           // Select P3.1(OA2O) as OA function
    P3SEL1 |= BIT1;                           // Select P3.1(OA2O) as OA function 
    SAC2DAT = 0xA88;                          // Set SAC2 DAC output = 1.646V
    SAC2DAC |= DACSREF_1 | DACEN;             // Select internal shared reference
                                              // and enable SAC2 DAC 
    SAC2OA |= NMUXEN + PMUXEN + NSEL_1 + PSEL_1; // Enable SAC2 negative and 
                                                 // positive input in DAC mode
    SAC2OA |= OAPM;                           // Select low speed and low power mode
    SAC2OA |= SACEN + OAEN;                   // Enable SAC2 and OA
    
    //Configure SAC0 to realize the PTC thermistor circuit  
    P1SEL0 |= BIT1 + BIT2 + BIT3;             // Select P1.1(OA0O/A1), P1.2(OA0-), 
                                              // and P1.3(OA0+) as OA function
    P1SEL1 |= BIT1 + BIT2 + BIT3;             // Select P1.1(OA0O), P1.2(OA0-), 
                                              // and P1.3(OA0+) as OA function
    SAC0OA |= NMUXEN + PMUXEN;                // Enable SAC0 negative and positive 
                                              // input
    SAC0OA |= OAPM;                           // Select low speed and low power mode
    SAC0OA |= SACEN + OAEN;                   // Enable SAC0 and OA
    
    
    //Configure ADC12 to sample the output of SAC0
    ADCCTL0 &= ~ADCENC;                       // Disable ADC
    ADCCTL0 = ADCSHT_2 | ADCON;               // ADCON, S&H=16 ADC clks, single
                                              // conversion
    ADCCTL1 = ADCSHP;                         // ADCCLK = MODOSC; sampling timer
    ADCCTL2 = ADCRES_2;                       // 12-bit conversion results
    ADCIE = ADCIE0;                           // Enable ADC conv complete interrupt
    ADCMCTL0 = ADCINCH_1 | ADCSREF_1;         // A1 ADC input select = OA0 output
                                              // ADC Voltage Reference = Internal 
                                              // Reference
  
    while(1)
    {
        ADCCTL0 |= ADCENC | ADCSC;            // Sampling and conversion start
        __bis_SR_register(LPM0_bits | GIE);   // Enter LPM0, ADC_ISR will force exit
        __no_operation();                     // For debug only
  
        // With 2.5V Reference: 1.67V/2.5V = 0.668
        // With a 12-bit ADC: 0.668 * 2^12 = 2736
        if ( adcResult > 2736 )               // OA output > 1.67V ( 25 deg C )
                                              // With 2.5V Reference:
        {
            P1OUT |= BIT0;                    // Set P1.0 LED on
        }
         else
        {
            P1OUT &= ~BIT0;                   // Clear P1.0 LED off
        }
    }

}

// ADC interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=ADC_VECTOR
__interrupt void ADC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC_VECTOR))) ADC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(ADCIV, ADCIV_ADCIFG))
    {
        case ADCIV_NONE:
            break;
        case ADCIV_ADCOVIFG:
            break;
        case ADCIV_ADCTOVIFG:
            break;
        case ADCIV_ADCHIIFG:
            break;
        case ADCIV_ADCLOIFG:
            break;
        case ADCIV_ADCINIFG:
            break;
        case ADCIV_ADCIFG:
            adcResult = ADCMEM0;                 // Read ADC memory
            __bic_SR_register_on_exit(LPM0_bits);// Exit from LPM
            break;
        default:
            break;
    }
}

